//+------------------------------------------------------------------+
//|                                                      MACDSAR.mq4 |
//|                                     Copyright 2012, FORTRADER.RU |
//|                                              http://fortrader.ru |
//+------------------------------------------------------------------+
#property copyright "Copyright 2012, FORTRADER.RU"
#property link      "http://fortrader.ru"

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1 Lime

#property  indicator_color3  Silver
#property  indicator_color4  Red
#property  indicator_width3  2
//---- input parameters
extern double    Step=0.01;
extern double    Maximum=0.1;

extern int FastEMA=5;
extern int SlowEMA=8;
extern int SignalSMA=9;
//---- buffers
double SarBuffer[];
double     MacdBuffer[];
double     SignalBuffer[];
//----
int    save_lastreverse;
bool   save_dirlong;
double save_start;
double save_last_high;
double save_last_low;
double save_ep;
double save_sar;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
//---- indicators
   SetIndexStyle(0,DRAW_ARROW);
   SetIndexArrow(0,159);
   SetIndexBuffer(0,SarBuffer);
   IndicatorDigits(MarketInfo(Symbol(),MODE_DIGITS));
   
   //---- drawing settings
   SetIndexStyle(2,DRAW_HISTOGRAM);
   SetIndexStyle(3,DRAW_LINE);
   SetIndexDrawBegin(3,SignalSMA);
   IndicatorDigits(Digits+1);
//---- indicator buffers mapping
   SetIndexBuffer(2,MacdBuffer);
   SetIndexBuffer(3,SignalBuffer);
//---- name for DataWindow and indicator subwindow label
   IndicatorShortName("MACD("+FastEMA+","+SlowEMA+","+SignalSMA+")");
   SetIndexLabel(2,"MACD");
   SetIndexLabel(3,"Signal");
//---- initialization done

//----
   return(0);
  }
//+------------------------------------------------------------------+
//| save values to fast calculations on next calls                   |
//+------------------------------------------------------------------+
void SaveLastReverse(int last,bool dir,double start,double low,double high,double ep,double sar)
  {
   save_lastreverse=last;
   save_dirlong=dir;
   save_start=start;
   save_last_low=low;
   save_last_high=high;
   save_ep=ep;
   save_sar=sar;
  }
//+------------------------------------------------------------------+
//| Parabolic Sell And Reverse system                                |
//+------------------------------------------------------------------+
int start()
  {
   Comment("    ForTrader.Ru");
   static bool first=true;
   bool   dirlong;
   double start,last_high,last_low;
   double ep,sar,price_low,price_high,price;
   int    i,counted_bars=IndicatorCounted();
  
   int limit;
   //---- last counted bar will be recounted
   if(counted_bars>0) counted_bars--;
   limit=Bars-counted_bars;
//---- macd counted in the 1-st buffer
   for(i=0; i<limit; i++)
      MacdBuffer[i]=iMA(NULL,0,FastEMA,0,MODE_EMA,PRICE_CLOSE,i)-iMA(NULL,0,SlowEMA,0,MODE_EMA,PRICE_CLOSE,i);
//---- signal line counted in the 2-nd buffer
   for(i=0; i<limit; i++)
      SignalBuffer[i]=iMAOnArray(MacdBuffer,Bars,SignalSMA,0,MODE_SMA,i);
//---- done
   i=0;
//---- nothing to calculate
   if(Bars<3) return(0);
//---- first calculation?
   if(counted_bars==0 || first)
     {
      //---- initial settings
      i=Bars-2;
      first=false;
      dirlong=true;
      start=Step;
      last_high=-10000000.0;
      last_low=10000000.0;
      save_lastreverse=0;
      sar=0;
      while(i>0)
        {
         save_lastreverse=i;
         price_low=iMACD(NULL,0,FastEMA,SlowEMA,SignalSMA,PRICE_CLOSE,MODE_MAIN,i);
         price_high=iMACD(NULL,0,FastEMA,SlowEMA,SignalSMA,PRICE_CLOSE,MODE_MAIN,i);
         if(last_low>price_low)
            last_low=price_low;
         if(last_high<price_high)
            last_high=price_high;
         if(price_high>iMACD(NULL,0,FastEMA,SlowEMA,SignalSMA,PRICE_CLOSE,MODE_MAIN,i+1) && price_low>iMACD(NULL,0,FastEMA,SlowEMA,SignalSMA,PRICE_CLOSE,MODE_MAIN,i+1))
            break;
         if(price_high<iMACD(NULL,0,FastEMA,SlowEMA,SignalSMA,PRICE_CLOSE,MODE_MAIN,i+1) && price_low<iMACD(NULL,0,FastEMA,SlowEMA,SignalSMA,PRICE_CLOSE,MODE_MAIN,i+1))
           {
            dirlong=false;
            break;
           }
         i--;
        }
      //---- initial zero
      int k=i;
      while(k<Bars)
        {
         SarBuffer[k]=0.0;
         k++;
        }
      //---- check further
      if(dirlong) { SarBuffer[i]=iMACD(NULL,0,FastEMA,SlowEMA,SignalSMA,PRICE_CLOSE,MODE_MAIN,i+1); ep=iMACD(NULL,0,FastEMA,SlowEMA,SignalSMA,PRICE_CLOSE,MODE_MAIN,i); }
      else        { SarBuffer[i]=iMACD(NULL,0,FastEMA,SlowEMA,SignalSMA,PRICE_CLOSE,MODE_MAIN,i+1); ep=iMACD(NULL,0,FastEMA,SlowEMA,SignalSMA,PRICE_CLOSE,MODE_MAIN,i+1); }
      i--;
     }
    else
     {
      //---- restore values from previous call to avoid full recalculation
      i=save_lastreverse;
      start=save_start;
      dirlong=save_dirlong;
      last_high=save_last_high;
      last_low=save_last_low;
      ep=save_ep;
      sar=save_sar;
     }
//----
   while(i>=0)
     {
      price_low=iMACD(NULL,0,FastEMA,SlowEMA,SignalSMA,PRICE_CLOSE,MODE_MAIN,i);
      price_high=iMACD(NULL,0,FastEMA,SlowEMA,SignalSMA,PRICE_CLOSE,MODE_MAIN,i);
      price=SarBuffer[i+1];
      //--- check for reverse
      if(dirlong && price_low<price)
        {
         SaveLastReverse(i+1,true,start,price_low,last_high,ep,sar);
         start=Step; dirlong=false;
         ep=price_low;
         last_low=price_low;
         SarBuffer[i]=last_high;
         i--;
         continue;
        }
      if(!dirlong && price_high>price)
        {
         SaveLastReverse(i+1,false,start,last_low,price_high,ep,sar);
         start=Step; dirlong=true;
         ep=price_high;
         last_high=price_high;
         SarBuffer[i]=last_low;
         i--;
         continue;
        }
      //--- calculate current value
      sar=price+start*(ep-price);
      //---- check long direction
      if(dirlong)
        {
         if(ep<price_high && (start+Step)<=Maximum)
            start+=Step;
         if(price_high<iMACD(NULL,0,FastEMA,SlowEMA,SignalSMA,PRICE_CLOSE,MODE_MAIN,i+1) && i==Bars-2)
            sar=price;
         if(sar>iMACD(NULL,0,FastEMA,SlowEMA,SignalSMA,PRICE_CLOSE,MODE_MAIN,i+1)) sar=iMACD(NULL,0,FastEMA,SlowEMA,SignalSMA,PRICE_CLOSE,MODE_MAIN,i+1);
         if(sar>iMACD(NULL,0,FastEMA,SlowEMA,SignalSMA,PRICE_CLOSE,MODE_MAIN,i+2)) sar=iMACD(NULL,0,FastEMA,SlowEMA,SignalSMA,PRICE_CLOSE,MODE_MAIN,i+2);
         if(sar>price_low)
           {
            SaveLastReverse(i+1,true,start,price_low,last_high,ep,sar);
            start=Step; dirlong=false; ep=price_low;
            last_low=price_low;
            SarBuffer[i]=last_high;
            i--;
            continue;
           }
         if(ep<price_high) { last_high=price_high; ep=price_high; }
        }
      else
        {
         if(ep>price_low && (start+Step)<=Maximum)
           start+=Step;
         if(price_low<iMACD(NULL,0,FastEMA,SlowEMA,SignalSMA,PRICE_CLOSE,MODE_MAIN,i+1) && i==Bars-2)
           sar=price;
         if(sar<iMACD(NULL,0,FastEMA,SlowEMA,SignalSMA,PRICE_CLOSE,MODE_MAIN,i+1)) sar=iMACD(NULL,0,FastEMA,SlowEMA,SignalSMA,PRICE_CLOSE,MODE_MAIN,i+1);
         if(sar<iMACD(NULL,0,FastEMA,SlowEMA,SignalSMA,PRICE_CLOSE,MODE_MAIN,i+2)) sar=iMACD(NULL,0,FastEMA,SlowEMA,SignalSMA,PRICE_CLOSE,MODE_MAIN,i+2);
         if(sar<price_high)
           {
            SaveLastReverse(i+1,false,start,last_low,price_high,ep,sar);
            start=Step; dirlong=true; ep=price_high;
            last_high=price_high;
            SarBuffer[i]=last_low;
            i--;
            continue;
           }
         if(ep>price_low) { last_low=price_low; ep=price_low; }
        }
      SarBuffer[i]=sar;
      i--;
     }
//----
   return(0);
  }
//+------------------------------------------------------------------+